# -*- coding:utf-8 -*-

import threading
import logging
from baidubce import exception
from baidubce.services import bos
from client import Client

logger = logging.getLogger(__name__)


class MultipartUploadHelper(object):
    MAX_PART_NUM = 10000

    def __init__(self, bucket_name, object_key):
        self.bucket_name = bucket_name
        self.object_key = object_key
        self.upload_id = -1
        self.part_list = []

    def init(self):
        try:
            self.upload_id = Client().bos_client.initiate_multipart_upload(
                    bucket_name = self.bucket_name, 
                    key = self.object_key).upload_id 
        except exception.BceError as e:
            logger.exception(
                    "initiate_multipart_upload occurs an exception.bucket_name: {0}, object_key: {1}, err_msg: {2}".format(str(self.bucket_name).encode("utf-8"), str(self.object_key).encode("utf-8"), e.message))
            raise e

    def upload_part(self, part_num, content, callback=None):
        logger.debug("upload_part part_num:{0}, content-length:{1}".format(part_num, len(content)))
        assert callback != None
        try:
            response = Client().bos_client.upload_part(
                    self.bucket_name, str(self.object_key), self.upload_id,
                    part_num, len(content), content)
        except exception.BceError as e:
            logger.exception(
                    "upload_part occurs an exception.bucket_name: {0}, object_key: {1}, upload_id: {2}, part_num: {3}, err_msg: {4}".format(str(self.bucket_name).encode("utf-8"), str(self.object_key).encode("utf-8"), self.upload_id, part_num, e.message))
            if callback is not None:
                callback(part_num, False)
            raise e
        except Exception as e:
            logger.exception("upload_part occurs Exception:{0}".format(str(e.message).encode("utf-8")))
            raise e

        logger.debug("upload_part success, part_num:{0}".format(part_num))
        self.part_list.append({"partNumber": part_num,"eTag": response.metadata.etag})
        callback(part_num, True)


    def complete_upload(self):
        # sort part_list
        self.part_list.sort(key=lambda part:part["partNumber"])
        try:
           response = Client().bos_client.complete_multipart_upload(bucket_name=self.bucket_name,
                                                                       key=str(self.object_key),
                                                                       upload_id=self.upload_id,
                                                                       part_list=self.part_list)
        except exception.BceError as e:
            logger.exception(
                    "complete_multipart_upload occurs an exception.bucket_name: {0}, object_key: {1}, upload_id: {2}, err_msg: {3}".format(str(self.bucket_name).encode("utf-8"), str(self.object_key).encode("utf-8"), self.upload_id, e.message))
            raise e
