# -*- coding:utf-8 -*-
"""
    test BosAbstractFileSystem
"""
import unittest
import mock

import sys
sys.path.append("../bosftp")
import   start_ftp_server
from     bosftp_config           import   BosftpCfg
from     bos_file_system         import   BosAbstractFileSystem
from     baidubce                import   exception
from     pyftpdlib.filesystems   import   FilesystemError
from     client                  import   Client
from     util                    import   random_string
class FaskResponse(object):
    """
        FaskResponse for return 
    """
    def __init__(self):
        self.content_length = 1024
        self.next_marker = None
        self.is_truncated = True
        self.contents = []
        self.common_prefixes = []

class BosAbstractFilesystemTest(unittest.TestCase):
    """
        BosAbstractFilesystemTest 
    """

    generate_pre_signed_url = Client().bos_client.generate_pre_signed_url
    put_object_from_string  = Client().bos_client.put_object_from_string
    copy_object             = Client().bos_client.copy_object
    list_objects            = Client().bos_client.list_objects

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_file_open(self):
        fs = BosAbstractFileSystem(u"/", None)
        fs.root = u"/"
        fs.cwd = u"/"
        Client().bos_client.generate_pre_signed_url = mock.Mock(return_value = "https://www.baidu.com/x.t")
        fs.open(u"object_key", "r")
        fs.open(u"object_key", "w")
        fs.isdir = mock.Mock(return_value = True)
        try:
            fs.open(u"object_ley", "R")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_file_operation filed")

        # recover function
        Client().bos_client.generate_pre_signed_url = self.generate_pre_signed_url

    def test_getsize(self):
        fs = BosAbstractFileSystem(u"/", None)
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, "testfile1", random_string(1024))
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, "testdir1/", random_string(1024))

        # normal file
        self.assertEqual("1024", fs.getsize(u"/testfile1"))
        # dir
        self.assertEqual(0, fs.getsize(u"/testdir1"))
        # none
        fs.isdir = mock.Mock(return_value = False)
        fs.isfile = mock.Mock(return_value = False)
    
        try:
            fs.getsize(u"none")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_getsize failed")

        Client().bos_client.delete_object(BosftpCfg().bucket, "testfile1")
        Client().bos_client.delete_object(BosftpCfg().bucket, "testdir1/")

    def test_chdir(self):
        fs = BosAbstractFileSystem(u"/", None)
        fs.isdir = mock.Mock(return_value = True)
        fs.chdir(u"/home")
        fs.isdir = mock.Mock(return_value = False)
        try:
            fs.chdir(u"dir")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_chdir failed")

    def test_mkdir(self):
        fs = BosAbstractFileSystem(u"/", None)
        fs.isdir = mock.Mock(return_value = True)
        try:
            fs.mkdir(u"ddd")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_mkdir failed")
        fs.isdir = mock.Mock(return_value = False)
        Client().bos_client.put_object_from_string = mock.Mock(return_value = FaskResponse())
        fs.mkdir(u"/xx")
        Client().bos_client.put_object_from_string = mock.Mock(side_effect = exception.BceError("BceError"))
        try:
            fs.mkdir(u"/xxd")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_mkdir failed")
        Client().bos_client.put_object_from_string = self.put_object_from_string

    def test_rename(self):
        fs = BosAbstractFileSystem(u"/", None)
        fs.rename(u"/src", u"/src")
        fs.isfile = mock.Mock(return_value = True)
        object_key = u"/test11"
        dest_key = u"/test11dest"
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, str(object_key).encode("utf-8")[1:], "123")
        fs.rename(object_key, dest_key)
        Client().bos_client.copy_object = mock.Mock(side_effect = exception.BceError("BceError"))
        try:
            fs.rename(u"src", u"dest")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_rename failed")
        fs.isfile = mock.Mock(return_value = False)
        fs.isdir = mock.Mock(return_value = True)

        try:
            fs.rename(u"src", u"dest")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_rename failed")
        
        # Client().bos_client.delete_object(BosftpCfg().bucket, str(object_key).encode("utf-8")[1:])
        Client().bos_client.copy_object = self.copy_object

    def test_listdir(self):
        fs = BosAbstractFileSystem(u"/", None)
        Client().bos_client.list_objects = mock.Mock(return_value = FaskResponse())
        self.assertEqual(0, len(fs.listdir(u"/")))
        self.assertEqual(0, len(fs.listdir(u"/xx")))
        Client().bos_client.list_objects = self.list_objects
        fs.listdir(u"/")

    def test_isdir(self):
        fs = BosAbstractFileSystem(u"/", None)
        file_name = "test111"
        dir_name = "test112/"
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, file_name, "123")
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, dir_name, "")
        u_file_name = u"/test111"
        u_dir_name = u"/test112"
        self.assertEqual(False, fs.isdir(u_file_name))
        self.assertEqual(True, fs.isdir(u_dir_name))
        # test fake dir
        Client().bos_client.list_objects = mock.Mock(return_value = FaskResponse)
        Client().bos_client.list_objects = self.list_objects

        Client().bos_client.delete_object(BosftpCfg().bucket, file_name)
        Client().bos_client.delete_object(BosftpCfg().bucket, dir_name)
        self.assertEqual(False, fs.isdir(u"/noneexixxxxist"))
        self.assertEqual(False, fs.isdir(u"ddd"))

    def test_rmdir(self):
        fs = BosAbstractFileSystem(u"/", None)
        dir_name = "test113/"
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, dir_name, "")
        fs.rmdir(u"/test113")
        fs.rmdir(u"/test113bucunzai")
        # Client().bos_client.delete_object(BosftpCfg().bucket, dir_name)

    def test_remove(self):
        fs = BosAbstractFileSystem(u"/", None)
        file_name = "test114"
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, file_name, "")
        fs.remove(u"/test114")

        try:
            fs.remove(u"/test114bucunzai")
        except FilesystemError as e:
            pass
        else:
            self.fail("test_remove failed")
        #Client().bos_client.delete_object(BosftpCfg().bucket, file_name)

    def test_lexists(self):
        fs = BosAbstractFileSystem(u"/", None)
        file_name = "test115"
        Client().bos_client.put_object_from_string(BosftpCfg().bucket, file_name, "")
        self.assertEqual(True, fs.lexists(u"/test115"))
        self.assertEqual(False, fs.lexists(u"/test115bucunzai"))
        self.assertEqual(False, fs.lexists(u"/bucunccczai"))
        Client().bos_client.delete_object(BosftpCfg().bucket, file_name)

    def test_format_ls(self):
        fs = BosAbstractFileSystem(u"/", None)
        list_keys = set()
        list_keys.add(("file",1, "Mon, 15 Oct 2018 12:08:28 GMT", "1"))
        list_keys.add(("dir",2, "Mon, 15 Oct 2018 13:08:28 GMT", "2"))
        list_keys.add(("file",3, "Mon, 15 Oct 2018 15:08:28 GMT", "3"))
        for i in fs.format_list("/", list_keys):
            print i

        for i in fs.format_mlsx("/", list_keys, None, None):
            print i

if __name__ == '__main__':
    unittest.main()
